@@ -242,50 +242,6 @@ function trigger_value2str($value = null) {
 	return _('Unknown');
 }
 
-function getParentHostsByTriggers($triggers) {
-	$hosts = [];
-	$triggerParent = [];
-
-	while (!empty($triggers)) {
-		foreach ($triggers as $tnum => $trigger) {
-			if ($trigger['templateid'] == 0) {
-				if (isset($triggerParent[$trigger['triggerid']])) {
-					foreach ($triggerParent[$trigger['triggerid']] as $triggerid => $state) {
-						$hosts[$triggerid] = $trigger['hosts'];
-					}
-				}
-				else {
-					$hosts[$trigger['triggerid']] = $trigger['hosts'];
-				}
-				unset($triggers[$tnum]);
-			}
-			else {
-				if (isset($triggerParent[$trigger['triggerid']])) {
-					if (!isset($triggerParent[$trigger['templateid']])) {
-						$triggerParent[$trigger['templateid']] = [];
-					}
-					$triggerParent[$trigger['templateid']][$trigger['triggerid']] = 1;
-					$triggerParent[$trigger['templateid']] += $triggerParent[$trigger['triggerid']];
-				}
-				else {
-					if (!isset($triggerParent[$trigger['templateid']])) {
-						$triggerParent[$trigger['templateid']] = [];
-					}
-					$triggerParent[$trigger['templateid']][$trigger['triggerid']] = 1;
-				}
-			}
-		}
-		$triggers = API::Trigger()->get([
-			'triggerids' => zbx_objectValues($triggers, 'templateid'),
-			'selectHosts' => ['hostid', 'host', 'name', 'status'],
-			'output' => ['triggerid', 'templateid'],
-			'filter' => ['flags' => null]
-		]);
-	}
-
-	return $hosts;
-}
-
 function get_trigger_by_triggerid($triggerid) {
 	$db_trigger = DBfetch(DBselect('SELECT t.* FROM triggers t WHERE t.triggerid='.zbx_dbstr($triggerid)));
 	if (!empty($db_trigger)) {
@@ -703,15 +659,24 @@ function replace_template_dependencies($deps, $hostid) {
  *
  * @param array  $groupids
  * @param string $application
- * @param int    $style            Table display style: either hosts on top, or host on the left side.
+ * @param int    $style                               Table display style: either hosts on top or on the left.
  * @param array  $host_options
  * @param array  $trigger_options
- * @param int    $show_suppressed  Whether to show items with suppressed problems.
+ * @param array  $problem_options
+ * @param int    $problem_options['min_severity']     (optional) Minimal problem severity.
+ * @param int    $problem_options['show_suppressed']  (optional) Whether to show triggers with suppressed problems.
+ * @param int    $problem_options['time_from']        (optional) The time starting from which the problems were created.
  *
  * @return array
  */
 function getTriggersOverviewData(array $groupids, $application, $style, array $host_options = [],
-		array $trigger_options = [], $show_suppressed = ZBX_PROBLEM_SUPPRESSED_FALSE) {
+		array $trigger_options = [], array $problem_options = []) {
+	$problem_options += [
+		'min_severity' => TRIGGER_SEVERITY_NOT_CLASSIFIED,
+		'show_suppressed' => ZBX_PROBLEM_SUPPRESSED_FALSE,
+		'time_from' => null
+	];
+
 	// fetch hosts
 	$hosts = API::Host()->get([
 		'output' => ['hostid', 'status'],
@@ -723,7 +688,7 @@ function getTriggersOverviewData(array $groupids, $application, $style, array $h
 
 	$hostids = array_keys($hosts);
 
-	$options = [
+	$trigger_options = [
 		'output' => ['triggerid', 'expression', 'description', 'url', 'value', 'priority', 'lastchange', 'flags',
 			'comments'],
 		'selectHosts' => ['hostid', 'name', 'status'],
@@ -744,10 +709,10 @@ function getTriggersOverviewData(array $groupids, $application, $style, array $h
 			'search' => ['name' => $application],
 			'preservekeys' => true
 		]);
-		$options['applicationids'] = array_keys($applications);
+		$trigger_options['applicationids'] = array_keys($applications);
 	}
 
-	$triggers = getTriggersWithActualSeverity($options, $show_suppressed);
+	$triggers = getTriggersWithActualSeverity($trigger_options, $problem_options);
 
 	$triggers = CMacrosResolverHelper::resolveTriggerUrls($triggers);
 
@@ -769,14 +734,22 @@ function getTriggersOverviewData(array $groupids, $application, $style, array $h
 /**
  * Get triggers data with priority set to highest priority of not resolved problems generated by this trigger.
  *
- * @param array $options          API options. Array 'output' should contain 'value', option 'preservekeys' should be
- *                                set to true.
- * @param int   $show_suppressed  Whether to show suppressed problems.
+ * @param array $trigger_options                     API options. Array 'output' should contain 'value', option
+ *                                                   'preservekeys' should be set to true.
+ * @param array $problem_options
+ * @param int   $problem_options['show_suppressed']  Whether to show triggers with suppressed problems.
+ * @param int   $problem_options['min_severity']     (optional) Minimal problem severity.
+ * @param int   $problem_options['time_from']        (optional) The time starting from which the problems were created.
  *
  * @return array
  */
-function getTriggersWithActualSeverity($options, $show_suppressed) {
-	$triggers = API::Trigger()->get($options);
+function getTriggersWithActualSeverity(array $trigger_options, array $problem_options) {
+	$problem_options += [
+		'min_severity' => TRIGGER_SEVERITY_NOT_CLASSIFIED,
+		'time_from' => null
+	];
+
+	$triggers = API::Trigger()->get($trigger_options);
 
 	$problem_triggerids = [];
 
@@ -792,7 +765,8 @@ function getTriggersWithActualSeverity($options, $show_suppressed) {
 		$problems = API::Problem()->get([
 			'output' => ['objectid', 'severity'],
 			'objectids' => $problem_triggerids,
-			'suppressed' => ($show_suppressed == ZBX_PROBLEM_SUPPRESSED_FALSE) ? false : null
+			'suppressed' => ($problem_options['show_suppressed'] == ZBX_PROBLEM_SUPPRESSED_FALSE) ? false : null,
+			'time_from' => $problem_options['time_from']
 		]);
 
 		$triggerids = zbx_objectValues($problems, 'objectid');
@@ -803,6 +777,12 @@ function getTriggersWithActualSeverity($options, $show_suppressed) {
 				$triggers[$problem['objectid']]['priority'] = $problem['severity'];
 			}
 		}
+
+		foreach ($triggers as $triggerid => $trigger) {
+			if ($trigger['priority'] < $problem_options['min_severity']) {
+				unset($triggers[$triggerid]);
+			}
+		}
 	}
 
 	return $triggers;
@@ -919,7 +899,7 @@ function getTriggersOverview(array $hosts, array $triggers, $pageFile, $viewMode
 	}
 	else {
 		// header
-		$header = [_('Host')];
+		$header = [_('Hosts')];
 
 		foreach ($data as $trigger_name => $trigger_data) {
 			foreach ($trigger_data as $trigger_hosts) {
@@ -1058,8 +1038,9 @@ function calculateAvailability($triggerId, $startTime, $endTime) {
 				' ORDER BY e.eventid DESC';
 		if ($row = DBfetch(DBselect($sql, 1))) {
 			$startValue = $row['value'];
-			$min = $startTime;
 		}
+
+		$min = $startTime;
 	}
 
 	$sql = 'SELECT COUNT(e.eventid) AS cnt,MIN(e.clock) AS min_clock,MAX(e.clock) AS max_clock'.
@@ -2375,6 +2356,259 @@ function makeTriggersHostsList(array $triggers_hosts) {
 	return $triggers_hosts;
 }
 
+/**
+ * Get parent templates for each given trigger.
+ *
+ * @param $array $triggers                  An array of triggers.
+ * @param string $triggers[]['triggerid']   ID of a trigger.
+ * @param string $triggers[]['templateid']  ID of parent template trigger.
+ * @param int    $flag                      Origin of the trigger (ZBX_FLAG_DISCOVERY_NORMAL or
+ *                                          ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ *
+ * @return array
+ */
+function getTriggerParentTemplates(array $triggers, $flag) {
+	$parent_triggerids = [];
+	$data = [
+		'links' => [],
+		'templates' => []
+	];
+
+	foreach ($triggers as $trigger) {
+		if ($trigger['templateid'] != 0) {
+			$parent_triggerids[$trigger['templateid']] = true;
+			$data['links'][$trigger['triggerid']] = ['triggerid' => $trigger['templateid']];
+		}
+	}
+
+	if (!$parent_triggerids) {
+		return $data;
+	}
+
+	$all_parent_triggerids = [];
+	$hostids = [];
+	if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+		$lld_ruleids = [];
+	}
+
+	do {
+		if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+			$db_triggers = API::TriggerPrototype()->get([
+				'output' => ['triggerid', 'templateid'],
+				'selectHosts' => ['hostid'],
+				'selectDiscoveryRule' => ['itemid'],
+				'triggerids' => array_keys($parent_triggerids)
+			]);
+		}
+		// ZBX_FLAG_DISCOVERY_NORMAL
+		else {
+			$db_triggers = API::Trigger()->get([
+				'output' => ['triggerid', 'templateid'],
+				'selectHosts' => ['hostid'],
+				'triggerids' => array_keys($parent_triggerids)
+			]);
+		}
+
+		$all_parent_triggerids += $parent_triggerids;
+		$parent_triggerids = [];
+
+		foreach ($db_triggers as $db_trigger) {
+			foreach ($db_trigger['hosts'] as $host) {
+				$data['templates'][$host['hostid']] = [];
+				$hostids[$db_trigger['triggerid']][] = $host['hostid'];
+			}
+
+			if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+				$lld_ruleids[$db_trigger['triggerid']] = $db_trigger['discoveryRule']['itemid'];
+			}
+
+			if ($db_trigger['templateid'] != 0) {
+				if (!array_key_exists($db_trigger['templateid'], $all_parent_triggerids)) {
+					$parent_triggerids[$db_trigger['templateid']] = true;
+				}
+
+				$data['links'][$db_trigger['triggerid']] = ['triggerid' => $db_trigger['templateid']];
+			}
+		}
+	}
+	while ($parent_triggerids);
+
+	foreach ($data['links'] as &$parent_trigger) {
+		$parent_trigger['hostids'] = array_key_exists($parent_trigger['triggerid'], $hostids)
+			? $hostids[$parent_trigger['triggerid']]
+			: [0];
+
+		if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+			$parent_trigger['lld_ruleid'] = array_key_exists($parent_trigger['triggerid'], $lld_ruleids)
+				? $lld_ruleids[$parent_trigger['triggerid']]
+				: 0;
+		}
+	}
+	unset($parent_trigger);
+
+	$db_templates = $data['templates']
+		? API::Template()->get([
+			'output' => ['name'],
+			'templateids' => array_keys($data['templates']),
+			'preservekeys' => true
+		])
+		: [];
+
+	$rw_templates = $db_templates
+		? API::Template()->get([
+			'output' => [],
+			'templateids' => array_keys($db_templates),
+			'editable' => true,
+			'preservekeys' => true
+		])
+		: [];
+
+	$data['templates'][0] = [];
+
+	foreach ($data['templates'] as $hostid => &$template) {
+		$template = array_key_exists($hostid, $db_templates)
+			? [
+				'hostid' => $hostid,
+				'name' => $db_templates[$hostid]['name'],
+				'permission' => array_key_exists($hostid, $rw_templates) ? PERM_READ_WRITE : PERM_READ
+			]
+			: [
+				'hostid' => $hostid,
+				'name' => _('Inaccessible template'),
+				'permission' => PERM_DENY
+			];
+	}
+	unset($template);
+
+	return $data;
+}
+
+/**
+ * Returns a template prefix for selected trigger.
+ *
+ * @param string $triggerid
+ * @param array  $parent_templates  The list of the templates, prepared by getTriggerParentTemplates() function.
+ * @param int    $flag              Origin of the trigger (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ *
+ * @return array|null
+ */
+function makeTriggerTemplatePrefix($triggerid, array $parent_templates, $flag) {
+	if (!array_key_exists($triggerid, $parent_templates['links'])) {
+		return null;
+	}
+
+	while (array_key_exists($parent_templates['links'][$triggerid]['triggerid'], $parent_templates['links'])) {
+		$triggerid = $parent_templates['links'][$triggerid]['triggerid'];
+	}
+
+	$templates = [];
+	foreach ($parent_templates['links'][$triggerid]['hostids'] as $hostid) {
+		$templates[] = $parent_templates['templates'][$hostid];
+	}
+
+	CArrayHelper::sort($templates, ['name']);
+
+	$list = [];
+
+	foreach ($templates as $template) {
+		if ($template['permission'] == PERM_READ_WRITE) {
+			if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+				$url = (new CUrl('trigger_prototypes.php'))
+					->setArgument('parent_discoveryid', $parent_templates['links'][$triggerid]['lld_ruleid']);
+			}
+			// ZBX_FLAG_DISCOVERY_NORMAL
+			else {
+				$url = (new CUrl('triggers.php'))
+					->setArgument('hostid', $template['hostid'])
+					->setArgument('filter_set', 1);
+			}
+
+			$name = (new CLink(CHtml::encode($template['name']), $url))->addClass(ZBX_STYLE_LINK_ALT);
+		}
+		else {
+			$name = new CSpan(CHtml::encode($template['name']));
+		}
+
+		$list[] = $name->addClass(ZBX_STYLE_GREY);
+		$list[] = ', ';
+	}
+
+	array_pop($list);
+	$list[] = NAME_DELIMITER;
+
+	return $list;
+}
+
+/**
+ * Returns a list of trigger templates.
+ *
+ * @param string $triggerid
+ * @param array  $parent_templates  The list of the templates, prepared by getTriggerParentTemplates() function.
+ * @param int    $flag              Origin of the trigger (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ *
+ * @return array
+ */
+function makeTriggerTemplatesHtml($triggerid, array $parent_templates, $flag) {
+	$list = [];
+
+	while (array_key_exists($triggerid, $parent_templates['links'])) {
+		$list_item = [];
+		$templates = [];
+
+		foreach ($parent_templates['links'][$triggerid]['hostids'] as $hostid) {
+			$templates[] = $parent_templates['templates'][$hostid];
+		}
+
+		$show_parentheses = (count($templates) > 1 && $list);
+
+		if ($show_parentheses) {
+			CArrayHelper::sort($templates, ['name']);
+			$list_item[] = '(';
+		}
+
+		foreach ($templates as $template) {
+			if ($template['permission'] == PERM_READ_WRITE) {
+				if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+					$url = (new CUrl('trigger_prototypes.php'))
+						->setArgument('form', 'update')
+						->setArgument('triggerid', $parent_templates['links'][$triggerid]['triggerid'])
+						->setArgument('parent_discoveryid', $parent_templates['links'][$triggerid]['lld_ruleid']);
+				}
+				// ZBX_FLAG_DISCOVERY_NORMAL
+				else {
+					$url = (new CUrl('triggers.php'))
+						->setArgument('form', 'update')
+						->setArgument('triggerid', $parent_templates['links'][$triggerid]['triggerid'])
+						->setArgument('hostid', $template['hostid']);
+				}
+
+				$name = new CLink(CHtml::encode($template['name']), $url);
+			}
+			else {
+				$name = (new CSpan(CHtml::encode($template['name'])))->addClass(ZBX_STYLE_GREY);
+			}
+
+			$list_item[] = $name;
+			$list_item[] = ', ';
+		}
+		array_pop($list_item);
+
+		if ($show_parentheses) {
+			$list_item[] = ')';
+		}
+
+		array_unshift($list, $list_item, '&nbsp;&rArr;&nbsp;');
+
+		$triggerid = $parent_templates['links'][$triggerid]['triggerid'];
+	}
+
+	if ($list) {
+		array_pop($list);
+	}
+
+	return $list;
+}
+
 /**
  * Check if user has read permissions for triggers.
  *
