@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,9 +27,19 @@ require_once dirname(__FILE__).'/include/blocks.inc.php';
 
 $page['title'] = _('Custom screens');
 $page['file'] = 'screens.php';
-$page['scripts'] = ['class.calendar.js', 'gtlc.js', 'flickerfreescreen.js', 'class.svg.canvas.js', 'class.svg.map.js'];
+$page['scripts'] = [
+	'class.calendar.js',
+	'gtlc.js',
+	'flickerfreescreen.js',
+	'class.svg.canvas.js',
+	'class.svg.map.js',
+	'layout.mode.js'
+];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
+CView::$has_web_layout_mode = true;
+$page['web_layout_mode'] = CView::getLayoutMode();
+
 define('ZBX_PAGE_DO_JS_REFRESH', 1);
 
 ob_start();
@@ -37,20 +47,19 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'groupid' =>	[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
-	'hostid' =>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
-	'tr_groupid' =>	[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
-	'tr_hostid' =>	[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
-	'elementid' =>	[T_ZBX_INT, O_OPT, P_SYS|P_NZERO, DB_ID, null],
-	'screenname' =>	[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'step' =>		[T_ZBX_INT, O_OPT, P_SYS,	BETWEEN(0, 65535), null],
-	'period' =>		[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
-	'stime' =>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'isNow' =>		[T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null],
-	'reset' =>		[T_ZBX_STR, O_OPT, P_SYS,	IN('"reset"'), null],
-	'fullscreen' =>	[T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'), null]
+	'groupid' =>	[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,		null],
+	'hostid' =>		[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,		null],
+	'tr_groupid' =>	[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,		null],
+	'tr_hostid' =>	[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,		null],
+	'elementid' =>	[T_ZBX_INT,			O_OPT, P_SYS|P_NZERO, DB_ID, null],
+	'screenname' =>	[T_ZBX_STR,			O_OPT, P_SYS,	null,		null],
+	'step' =>		[T_ZBX_INT,			O_OPT, P_SYS,	BETWEEN(0, 65535), null],
+	'from' =>		[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,		null],
+	'to' =>			[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,	null,		null],
+	'reset' =>		[T_ZBX_STR,			O_OPT, P_SYS,	IN('"reset"'), null]
 ];
 check_fields($fields);
+validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 
 /*
  * Permissions
@@ -79,12 +88,7 @@ if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
 /*
  * Display
  */
-$data = [
-	'fullscreen' => $_REQUEST['fullscreen'],
-	'period' => getRequest('period'),
-	'stime' => getRequest('stime'),
-	'isNow' => getRequest('isNow')
-];
+$data = [];
 
 $options = [
 	'output' => ['screenid', 'name']
@@ -123,6 +127,17 @@ else {
 		'screenids' => [$data['screen']['screenid']],
 		'editable' => true
 	]);
+	$data['active_tab'] = CProfile::get('web.screens.filter.active', 1);
+
+	$timeselector_options = [
+		'profileIdx' => 'web.screens.filter',
+		'profileIdx2' => $data['screen']['screenid'],
+		'from' => getRequest('from'),
+		'to' => getRequest('to')
+	];
+	updateTimeSelectorPeriod($timeselector_options);
+
+	$data += $timeselector_options;
 }
 ob_end_flush();
 
